package Test::Regression;
use warnings;
use strict;

our $VERSION = 0.02;

use base 'Test::Builder::Module';

use Exporter 'import';
our @EXPORT = qw/ no_regression_ok /;

my $tb = __PACKAGE__->builder;

sub no_regression_ok ($&;$) {
    my ($todo, $code, $msg ) = @_;
    defined $todo or $todo = "";
    defined $msg  or $msg  = "";

    local $::TODO; # Test::Builder looks for $main::TODO
    if ( $todo =~ s/^TODO(?:\s+|$)// ) {
        $::TODO = $todo || 'XXX';
    }
    else {
        $msg .= length $msg ? " $todo" : $todo
            if length $todo;
    }

    my $result = eval { $code->() };
    my $error = $@;

    if ( $error ) {
        $tb->diag( "[Regression DIE] $error" );
        return $tb->ok( 0, $msg );
    }

    if ( ! $result ) {
        $tb->diag( "[Regression FAIL]" );
        return $tb->ok( 0, $msg );
    }

    return $tb->ok( 1, $msg );
}

1;

__END__

=head1 NAME

Test::Regression - Provide no_regression_ok() with TODO magic

=head1 SYNOPSIS

    use Test::More tests => 2;
    use Test::Regression;

    # Before fixing the bug
    no_regression_ok(
        "TODO Ticket XXX: subject",
        sub { return 0 },
        "Testname"
    );

    # After fixing the bug
    no_regression_ok(
        "Ticket XXX: subject",
        sub { return 1 },
        "Testname"
    );


=head1 DESCRIPTION

=head2 my $ok = no_regression_ok( $todo, $code_ref, $testname )

This function runs the code within an C<eval { }>. If an exception was
thrown the test fails. If no exception was found it takes the result
of the code to be the result of the test.

=head3 Arguments

=over 4

=item B<$todo>, Specify this item if the test must be marked TODO,
else leave empty.

=item B<$code_ref>, The code that must be run, it should return the
test-status.

=item B<$testname>, Optional testname.

=back

=head3 Return value

True or false.

=head1 STUFF

(c) MMIX - Abe Timmerman <abeltje@cpan.org>

=cut
